import copy
from squid.structures.atom import Atom
from squid.structures.molecule import Molecule
from squid.structures.topology import Connector


def get_unit_test_structures():
    m1a = Molecule([
        Atom("H", 0, 0, 0),
        Atom("He", 1, 0, 0),
        Atom("F", 2, 0, 0),
        Atom("H", 3, 0, 0),
        Atom("H", 4, 0, 0),
        Atom("H", 5, 0, 0),
    ])
    m1b = Molecule([
        Atom("H", 0, 0, 0),
        Atom("He", 1, 0, 0),
        Atom("F", 2, 0, 0),
        Atom("H", 3, 0, 0),
        Atom("H", 4, 0, 0),
        Atom("H", 5, 0, 0),
    ])
    m2 = Molecule([
        Atom("O", 0, 1, 0),
        Atom("Si", 1, 0, 0),
        Atom("O", 2, 1, 0),
        Atom("H", 3, 1, 0),
        Atom("H", 3, 2, 0),
    ])
    m2.bonds = [
        Connector((m2.atoms[0], m2.atoms[1])),
        Connector((m2.atoms[2], m2.atoms[1])),
        Connector((m2.atoms[2], m2.atoms[3])),
        Connector((m2.atoms[2], m2.atoms[4])),
    ]

    atoms = [
        Atom("C", -1.54846, -0.62372, -0.23277),
        Atom("C", -2.36075, -1.83030, 0.22901),
        Atom("C", -0.09058, -0.72742, 0.20628),
        Atom("H", -1.99055, 0.29459, 0.16994),
        Atom("H", -1.59582, -0.54877, -1.32609),
        Atom("C", 0.54389, -2.03745, -0.25281),
        Atom("H", -0.03335, -0.65785, 1.29946),
        Atom("H", 0.47714, 0.11843, -0.19774),
        Atom("C", -0.26907, -3.24874, 0.19654),
        Atom("H", 1.56326, -2.11120, 0.14290),
        Atom("H", 0.62510, -2.03877, -1.34664),
        Atom("C", -1.73066, -3.14223, -0.23062),
        Atom("H", -3.38310, -1.75673, -0.15876),
        Atom("H", -2.43273, -1.82396, 1.32351),
        Atom("H", -1.79709, -3.21480, -1.32304),
        Atom("H", -2.29648, -3.98613, 0.18023),
        Atom("H", -0.21477, -3.33864, 1.28832),
        Atom("H", 0.17038, -4.16187, -0.22089)
    ]
    bonds = [
        Connector((atoms[11 - 1], atoms[6 - 1])),
        Connector((atoms[5 - 1], atoms[1 - 1])),
        Connector((atoms[15 - 1], atoms[12 - 1])),
        Connector((atoms[6 - 1], atoms[10 - 1])),
        Connector((atoms[6 - 1], atoms[9 - 1])),
        Connector((atoms[6 - 1], atoms[3 - 1])),
        Connector((atoms[1 - 1], atoms[4 - 1])),
        Connector((atoms[1 - 1], atoms[3 - 1])),
        Connector((atoms[1 - 1], atoms[2 - 1])),
        Connector((atoms[12 - 1], atoms[16 - 1])),
        Connector((atoms[12 - 1], atoms[9 - 1])),
        Connector((atoms[12 - 1], atoms[2 - 1])),
        Connector((atoms[18 - 1], atoms[9 - 1])),
        Connector((atoms[8 - 1], atoms[3 - 1])),
        Connector((atoms[13 - 1], atoms[2 - 1])),
        Connector((atoms[9 - 1], atoms[17 - 1])),
        Connector((atoms[3 - 1], atoms[7 - 1])),
        Connector((atoms[2 - 1], atoms[14 - 1]))
    ]
    chex = Molecule(atoms, bonds)
    copied_chex = copy.deepcopy(chex)

    return m1a, m1b, m2, chex, copied_chex


def get_test_frames():
    return [
        [
            Atom("H", *[-2.26963303, -2.36704362, -0.14561799]),
            Atom("H", *[-1.94229394, -0.88104468, 0.39481497]),
            Atom("H", *[-1.11230924, -2.88690974, -1.07363953]),
            Atom("H", *[-3.2549483, -2.32405029, -0.6914038]),
            Atom("H", *[-2.43112735, -2.96503896, 0.72038486]),
            Atom("H", *[0.21522239, -2.85563558, -0.36978078]),
            Atom("H", *[-1.12425159, -2.26723255, -1.92971767]),
            Atom("H", *[-1.40879998, -3.87772082, -1.35180643]),
            Atom("H", *[0.55496435, -1.46627237, 0.25909863]),
            Atom("H", *[1.03771586, -3.16134563, -1.03222485]),
            Atom("H", *[0.19050639, -3.58945941, 0.43687997]),
            Atom("H", *[-0.57141413, -0.89887877, 1.15721151]),
            Atom("H", *[-2.74477803, -0.60671529, 0.99781857]),
            Atom("H", *[-1.92287373, -0.21480419, -0.39959227]),
            Atom("H", *[-0.66496374, -1.49219559, 1.98443621]),
            Atom("H", *[-0.34512307, 0.07122213, 1.35801967]),
            Atom("H", *[0.73070611, -0.79514009, -0.60092696]),
            Atom("H", *[1.42943804, -1.45377973, 0.78180727])
        ],
        [
            Atom("H", *[-0.14117639, -1.32090008, 2.38874369]),
            Atom("H", *[0.30784922, 0.09897177, 2.15414106]),
            Atom("H", *[-0.87366596, -1.98731359, 1.2396093]),
            Atom("H", *[-0.7980844, -1.32745952, 3.32222966]),
            Atom("H", *[0.80055342, -1.94939008, 2.62549419]),
            Atom("H", *[-0.0082457, -1.85006716, -0.00208363]),
            Atom("H", *[-1.74544261, -1.48916842, 1.06677651]),
            Atom("H", *[-1.06053755, -2.97782029, 1.46125499]),
            Atom("H", *[0.43700512, -0.48253486, -0.3054071]),
            Atom("H", *[-0.58182919, -2.26199405, -0.83575506]),
            Atom("H", *[0.8883596, -2.50876943, 0.12266733]),
            Atom("H", *[1.11009024, 0.21093227, 0.82154169]),
            Atom("H", *[0.88416416, 0.54312981, 2.99949398]),
            Atom("H", *[-0.57883228, 0.73890661, 2.02086625]),
            Atom("H", *[2.13768291, -0.29448082, 1.02081099]),
            Atom("H", *[1.3024384, 1.24007626, 0.66948351]),
            Atom("H", *[-0.49690934, 0.16404258, -0.64618801]),
            Atom("H", *[1.11053544, -0.47653638, -1.23913764])
        ],
        [
            Atom("H", *[-0.30206183, -0.05000845, 2.31324809]),
            Atom("H", *[0.18341914, 0.07809875, 0.8307595]),
            Atom("H", *[0.80201968, -0.67020984, 3.18986595]),
            Atom("H", *[-1.12830003, -0.74625663, 2.22905917]),
            Atom("H", *[-0.6155309, 0.93881486, 2.64887795]),
            Atom("H", *[2.02620733, 0.19340098, 3.17891862]),
            Atom("H", *[1.02523421, -1.7207875, 2.92553559]),
            Atom("H", *[0.35844152, -0.70302527, 4.26658191]),
            Atom("H", *[2.60949215, 0.29144272, 1.78839676]),
            Atom("H", *[2.78163399, -0.36498126, 3.7782423]),
            Atom("H", *[1.85286812, 1.17271578, 3.64313093]),
            Atom("H", *[1.57435159, 0.92061554, 0.83535941]),
            Atom("H", *[-0.5172491, 0.5371092, 0.18200506]),
            Atom("H", *[0.3981551, -0.99096926, 0.40612919]),
            Atom("H", *[1.30190906, 1.8974868, 1.06302375]),
            Atom("H", *[1.93587596, 0.88242036, -0.23716487]),
            Atom("H", *[2.92070099, -0.62835916, 1.39257975]),
            Atom("H", *[3.48830198, 0.9621878, 1.74754853])
        ],
        [
            Atom("H", *[1.38562271, 0.52951049, 1.95705161]),
            Atom("H", *[0.9228094, 1.81783568, 2.67768278]),
            Atom("H", *[2.88079999, 0.34339826, 1.99505851]),
            Atom("H", *[1.02745621, 0.63799891, 0.82904991]),
            Atom("H", *[0.84632079, -0.31903248, 2.33603793]),
            Atom("H", *[3.34933863, 0.36282811, 3.38293624]),
            Atom("H", *[3.40359034, 1.18347335, 1.4104239]),
            Atom("H", *[3.18193106, -0.59558861, 1.48491942]),
            Atom("H", *[2.90591328, 1.69884894, 4.11612943]),
            Atom("H", *[4.43266886, 0.33793584, 3.4189374]),
            Atom("H", *[2.96717855, -0.50875237, 4.01428089]),
            Atom("H", *[1.45048204, 1.89608193, 4.07805531]),
            Atom("H", *[-0.17730276, 1.91224725, 2.59591116]),
            Atom("H", *[1.30117695, 2.70618499, 2.10662005]),
            Atom("H", *[0.92313422, 1.08952611, 4.61094006]),
            Atom("H", *[1.16782687, 2.78137196, 4.52480202]),
            Atom("H", *[3.41893695, 2.55241483, 3.68692083]),
            Atom("H", *[3.17921318, 1.57356685, 5.19681413])
        ],
        [
            Atom("H", *[3.74939783, 1.06628306, 4.86362791]),
            Atom("H", *[4.21070002, 1.3299586, 3.44258888]),
            Atom("H", *[3.21272943, 2.26055835, 5.4800097]),
            Atom("H", *[4.607496, 0.65204113, 5.44911678]),
            Atom("H", *[2.96947645, 0.27686539, 4.78872763]),
            Atom("H", *[2.13098563, 2.925936, 4.69184808]),
            Atom("H", *[4.04967476, 3.01827123, 5.63042966]),
            Atom("H", *[2.82704908, 2.09153655, 6.4590055]),
            Atom("H", *[2.51225899, 3.20615127, 3.22605253]),
            Atom("H", *[1.82069037, 3.89332599, 5.08468744]),
            Atom("H", *[1.23502073, 2.28193416, 4.65575348]),
            Atom("H", *[3.05517368, 1.91780845, 2.51134957]),
            Atom("H", *[4.5090649, 0.39940226, 2.93014395]),
            Atom("H", *[5.09162878, 1.97547007, 3.44680246]),
            Atom("H", *[2.24017863, 1.23596189, 2.4355485]),
            Atom("H", *[3.51947379, 2.18039666, 1.56822542]),
            Atom("H", *[3.36773416, 3.96729812, 3.18052048]),
            Atom("H", *[1.67246089, 3.56780083, 2.69437584])
        ],
        [
            Atom("H", *[5.26961897, 3.63254149, 5.85056121]),
            Atom("H", *[4.1497009, 4.56736204, 5.32855714]),
            Atom("H", *[4.81468433, 2.1734174, 5.89868643]),
            Atom("H", *[5.74964378, 3.95684729, 6.77592983]),
            Atom("H", *[6.07012919, 3.66896347, 5.06349741]),
            Atom("H", *[4.10899782, 1.64937734, 4.69336448]),
            Atom("H", *[4.11291742, 2.1418218, 6.81135771]),
            Atom("H", *[5.68086828, 1.4855467, 6.21786566]),
            Atom("H", *[2.98419028, 2.6124367, 4.29233615]),
            Atom("H", *[3.79952477, 0.68085802, 4.86477401]),
            Atom("H", *[4.84002657, 1.62543602, 3.88200003]),
            Atom("H", *[3.50665937, 4.04610251, 4.1241421]),
            Atom("H", *[4.55850426, 5.52302615, 5.19002568]),
            Atom("H", *[3.4165901, 4.56163479, 6.2394232]),
            Atom("H", *[4.17198947, 4.16660199, 3.23949447]),
            Atom("H", *[2.57385354, 4.71734296, 3.89526838]),
            Atom("H", *[2.21652968, 2.59199634, 4.96001189]),
            Atom("H", *[2.58969806, 2.28069287, 3.34987429])
        ],
        [
            Atom("H", *[4.42433396, 3.19173676, 5.17106291]),
            Atom("H", *[4.1246707, 3.25171238, 6.61429172]),
            Atom("H", *[4.89243886, 4.54300822, 4.67671789]),
            Atom("H", *[3.52804132, 2.90031887, 4.64162367]),
            Atom("H", *[5.16530775, 2.42844509, 4.98178086]),
            Atom("H", *[6.15816465, 5.00193201, 5.38882241]),
            Atom("H", *[4.11731671, 5.27639889, 4.7527745]),
            Atom("H", *[5.19283408, 4.52897882, 3.57493544]),
            Atom("H", *[5.83831623, 5.01520455, 6.90853174]),
            Atom("H", *[6.43576144, 6.06635232, 5.09470107]),
            Atom("H", *[7.01369844, 4.34923889, 5.24094207]),
            Atom("H", *[5.337499, 3.70600341, 7.41856225]),
            Atom("H", *[3.83395408, 2.2747387, 6.92328547]),
            Atom("H", *[3.30127953, 3.9112786, 6.7808129]),
            Atom("H", *[6.16598329, 2.98260345, 7.38907524]),
            Atom("H", *[5.03713805, 3.81569194, 8.48058879]),
            Atom("H", *[5.09159855, 5.87445663, 7.19086755]),
            Atom("H", *[6.7574713, 5.34150692, 7.46387403])
        ],
        [
            Atom("H", *[6.84751078, 6.35310321, 7.02806697]),
            Atom("H", *[7.24021701, 5.2608474, 7.93625162]),
            Atom("H", *[6.21394854, 5.87361473, 5.78281163]),
            Atom("H", *[7.73337388, 6.95294256, 6.72200093]),
            Atom("H", *[6.18380415, 7.02692117, 7.50009586]),
            Atom("H", *[5.04481365, 4.9859312, 6.07393077]),
            Atom("H", *[6.93820589, 5.30358026, 5.1292396]),
            Atom("H", *[5.88338137, 6.61522987, 5.09387414]),
            Atom("H", *[5.41997937, 3.82745128, 7.04247712]),
            Atom("H", *[4.65581677, 4.51479226, 5.1403208]),
            Atom("H", *[4.21186182, 5.48098648, 6.48970983]),
            Atom("H", *[6.01971912, 4.35689466, 8.29016542]),
            Atom("H", *[7.70214766, 5.73863166, 8.88599335]),
            Atom("H", *[8.03841905, 4.65223424, 7.52218971]),
            Atom("H", *[5.35414441, 4.93383859, 8.82755799]),
            Atom("H", *[6.41263489, 3.53117743, 8.87965372]),
            Atom("H", *[6.09645369, 3.11899286, 6.5014111]),
            Atom("H", *[4.52063346, 3.24735799, 7.32358133])
        ],
        [
            Atom("H", *[5.87163247, 5.93736438, 7.32070889]),
            Atom("H", *[5.88265181, 6.55631305, 8.66034252]),
            Atom("H", *[7.2239428, 6.15267557, 6.60044633]),
            Atom("H", *[5.03893269, 6.36530444, 6.73827217]),
            Atom("H", *[5.76110174, 4.80560788, 7.39430688]),
            Atom("H", *[8.41742556, 5.67986498, 7.42534686]),
            Atom("H", *[7.32485171, 7.2737427, 6.3271934]),
            Atom("H", *[7.14627261, 5.64763188, 5.59168809]),
            Atom("H", *[8.4017468, 6.27251004, 8.78908454]),
            Atom("H", *[9.28160853, 5.9361427, 6.90324544]),
            Atom("H", *[8.40422381, 4.57217682, 7.48120288]),
            Atom("H", *[7.05680173, 6.09124022, 9.49990356]),
            Atom("H", *[4.95970845, 6.27606136, 9.18215793]),
            Atom("H", *[5.92468396, 7.6621071, 8.66254546]),
            Atom("H", *[6.91701692, 4.94397815, 9.75262021]),
            Atom("H", *[7.08772099, 6.52742292, 10.46917852]),
            Atom("H", *[8.59072699, 7.34662867, 8.72088577]),
            Atom("H", *[9.15699083, 5.87235418, 9.3769635])
        ],
        [
            Atom("H", *[8.04096878, 6.26974573, 7.81966178]),
            Atom("H", *[7.78812106, 7.77616737, 7.75964098]),
            Atom("H", *[9.06890611, 5.92154374, 8.9056194]),
            Atom("H", *[8.48039468, 5.97310611, 6.83761113]),
            Atom("H", *[7.17232407, 5.75772192, 7.86544102]),
            Atom("H", *[8.48116416, 6.41637463, 10.26302803]),
            Atom("H", *[10.025956, 6.35030408, 8.73384837]),
            Atom("H", *[9.16305072, 4.80479221, 8.91424199]),
            Atom("H", *[8.19256709, 7.94388863, 10.27542034]),
            Atom("H", *[9.31360397, 6.226067, 11.03313557]),
            Atom("H", *[7.58932055, 5.86297253, 10.51627231]),
            Atom("H", *[7.23482611, 8.31100573, 9.10408375]),
            Atom("H", *[7.07694858, 7.96402401, 6.99335555]),
            Atom("H", *[8.73094654, 8.33702022, 7.54015612]),
            Atom("H", *[6.29714448, 7.85333945, 9.27072201]),
            Atom("H", *[7.11141357, 9.33860264, 8.99813436]),
            Atom("H", *[9.20787388, 8.48499618, 10.13647379]),
            Atom("H", *[7.77036198, 8.17370623, 11.19785154])
        ]
    ]


def get_THF():
    atoms = [
        Atom("C", -1.736497, -4.767184, -0.351683, label="81"),
        Atom("C", -0.498357, -3.928215, -0.165400, label="81"),
        Atom("C", 0.560821, -4.984799, 0.058288, label="124"),
        Atom("H", -0.570292, -3.213718, 0.658726, label="85"),
        Atom("H", -0.454341, -2.990383, -0.925335, label="85"),
        Atom("C", -1.198636, -5.959340, -1.113897, label="124"),
        Atom("H", -2.542882, -4.251468, -0.879709, label="85"),
        Atom("H", -2.117077, -5.088863, 0.624875, label="85"),
        Atom("O", 0.169308, -6.127693, -0.714742, label="122"),
        Atom("H", -1.212040, -5.779072, -2.193940, label="127"),
        Atom("H", -1.757611, -6.876188, -0.906203, label="127"),
        Atom("H", 0.600011, -5.290579, 1.109260, label="127"),
        Atom("H", 1.557569, -4.654603, -0.247261, label="127")
    ]

    bonds = [
        Connector((atoms[1 - 1], atoms[2 - 1])),
        Connector((atoms[2 - 1], atoms[3 - 1])),
        Connector((atoms[2 - 1], atoms[4 - 1])),
        Connector((atoms[2 - 1], atoms[5 - 1])),
        Connector((atoms[1 - 1], atoms[6 - 1])),
        Connector((atoms[1 - 1], atoms[7 - 1])),
        Connector((atoms[1 - 1], atoms[8 - 1])),
        Connector((atoms[9 - 1], atoms[6 - 1])),
        Connector((atoms[6 - 1], atoms[10 - 1])),
        Connector((atoms[6 - 1], atoms[11 - 1])),
        Connector((atoms[9 - 1], atoms[3 - 1])),
        Connector((atoms[3 - 1], atoms[12 - 1])),
        Connector((atoms[3 - 1], atoms[13 - 1]))
    ]
    return Molecule(atoms, bonds)
